@@ -1,10 +1,10 @@
 ;;; project-am.el --- A project management scheme based on automake files.
 
-;; Copyright (C) 1998, 1999, 2000, 2003, 2005, 2007, 2008, 2009, 2010, 2011, 2012
-;;   Free Software Foundation, Inc.
+;; Copyright (C) 1998-2000, 2003, 2005, 2007-2021 Free Software
+;; Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
-;; Version: 0.0.3
+;; Old-Version: 0.0.3
 ;; Keywords: project, make
 
 ;; This file is part of GNU Emacs.
@@ -20,7 +20,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 ;;
@@ -53,19 +53,19 @@
   )
 
 (defcustom project-am-compile-project-command nil
-  "*Default command used to compile a project."
+  "Default command used to compile a project."
   :group 'project-am
-  :type 'string)
+  :type '(choice (const nil) string))
 
 (defcustom project-am-compile-target-command (concat ede-make-command " -k %s")
-  "*Default command used to compile a project."
+  "Default command used to compile a project."
   :group 'project-am
   :type 'string)
 
 (defcustom project-am-debug-target-function 'gdb
-  "*Default Emacs command used to debug a target."
+  "Default Emacs command used to debug a target."
   :group 'project-am
-  :type 'sexp) ; make this be a list some day
+  :type 'function) ; make this be a list some day
 
 (defconst project-am-type-alist
   '(("bin" project-am-program "bin_PROGRAMS" t)
@@ -194,7 +194,7 @@ other meta-variable based on this name.")
   "Encode one makefile.")
 
 ;;; Code:
-(defmethod project-add-file ((ot project-am-target))
+(cl-defmethod project-add-file ((ot project-am-target))
   "Add the current buffer into a project.
 OT is the object target.  DIR is the directory to start in."
   (let* ((target (if ede-object (error "Already associated w/ a target")
@@ -205,7 +205,7 @@ OT is the object target.  DIR is the directory to start in."
 							 (oref amf targets))
 				      nil t))))
 	 ;; The input target might be new.  See if we can find it.
-	 (amf (ede-load-project-file (oref ot path)))
+	 (amf (ede-target-parent ot))
 	 (ot (object-assoc target 'name (oref amf targets)))
 	 (ofn (file-name-nondirectory (buffer-file-name))))
     (if (not ot)
@@ -214,7 +214,6 @@ OT is the object target.  DIR is the directory to start in."
 	       target (project-am-preferred-target-type (buffer-file-name)))))
     (ede-with-projectfile ot
       (makefile-move-to-macro (project-am-macro ot))
-      (ede-maybe-checkout)
       (makefile-end-of-command)
       (insert " " ofn)
       (makefile-fill-paragraph nil)
@@ -222,14 +221,10 @@ OT is the object target.  DIR is the directory to start in."
       (save-buffer))
     (setq ede-object ot)))
 
-(defmethod project-remove-file ((ot project-am-target) fnnd)
+(cl-defmethod project-remove-file ((ot project-am-target) fnnd)
   "Remove the current buffer from any project targets."
   (ede-with-projectfile ot
     (makefile-move-to-macro (project-am-macro ot))
-    (if (and buffer-read-only vc-mode
-	     (y-or-n-p "Checkout Makefile.am from VC? "))
-	(vc-toggle-read-only t))
-    (ede-maybe-checkout)
     (makefile-navigate-macro (concat " *" (regexp-quote (ede-name fnnd))))
     (replace-match "" t t nil 0)
     (makefile-fill-paragraph nil)
@@ -237,7 +232,7 @@ OT is the object target.  DIR is the directory to start in."
     (save-buffer))
   (setq ede-object nil))
 
-(defmethod project-edit-file-target ((obj project-am-target))
+(cl-defmethod project-edit-file-target ((obj project-am-target))
   "Edit the target associated w/ this file."
   (find-file (concat (oref obj path) "Makefile.am"))
   (goto-char (point-min))
@@ -245,7 +240,7 @@ OT is the object target.  DIR is the directory to start in."
   (if (= (point-min) (point))
       (re-search-forward (ede-target-name obj))))
 
-(defmethod project-new-target ((proj project-am-makefile)
+(cl-defmethod project-new-target ((proj project-am-makefile)
 			       &optional name type)
   "Create a new target named NAME.
 Argument TYPE is the type of target to insert.  This is a string
@@ -271,7 +266,6 @@ buffer being in order to provide a smart default target type."
     (if (not ot) (error "Error creating target object %S" ntype))
     (ede-with-projectfile ot
       (goto-char (point-min))
-      (ede-maybe-checkout)
       (makefile-next-dependency)
       (if (= (point) (point-min))
 	  (goto-char (point-max))
@@ -290,7 +284,7 @@ buffer being in order to provide a smart default target type."
 		(progn (forward-line -1)
 		       (end-of-line)
 		       (insert "\n"))
-	      ;; If the above search fails, thats ok.  We'd just want to be at
+	      ;; If the above search fails, that's ok.  We'd just want to be at
 	      ;; point-min anyway.
 	      )
 	    (makefile-insert-macro (car (cdr (cdr ntype))))))
@@ -306,7 +300,7 @@ buffer being in order to provide a smart default target type."
 ;;  This should be handled at the EDE level, calling a method of the
 ;; top most project.
 ;;
-(defmethod project-compile-project ((obj project-am-target) &optional command)
+(cl-defmethod project-compile-project ((obj project-am-target) &optional command)
   "Compile the entire current project.
 Argument COMMAND is the command to use when compiling."
   (require 'compile)
@@ -330,7 +324,7 @@ Argument COMMAND is the command to use when compiling."
   (let* ((default-directory (project-am-find-topmost-level default-directory)))
     (compile command)))
 
-(defmethod project-compile-project ((obj project-am-makefile)
+(cl-defmethod project-compile-project ((obj project-am-makefile)
 				    &optional command)
   "Compile the entire current project.
 Argument COMMAND is the command to use when compiling."
@@ -355,7 +349,7 @@ Argument COMMAND is the command to use when compiling."
   (let* ((default-directory (project-am-find-topmost-level default-directory)))
     (compile command)))
 
-(defmethod project-compile-target ((obj project-am-target) &optional command)
+(cl-defmethod project-compile-target ((obj project-am-target) &optional command)
   "Compile the current target.
 Argument COMMAND is the command to use for compiling the target."
   (require 'compile)
@@ -384,7 +378,7 @@ Argument COMMAND is the command to use for compiling the target."
   ;; We better be in the right place when compiling a specific target.
   (compile command))
 
-(defmethod project-debug-target ((obj project-am-objectcode))
+(cl-defmethod project-debug-target ((obj project-am-objectcode))
   "Run the current project target in a debugger."
   (let ((tb (get-buffer-create " *padt*"))
 	(dd (oref obj path))
@@ -401,7 +395,9 @@ Argument COMMAND is the command to use for compiling the target."
 	  (funcall project-am-debug-target-function cmd))
       (kill-buffer tb))))
 
-(defmethod project-run-target ((obj project-am-objectcode))
+(declare-function ede-shell-run-something "ede/shell")
+
+(cl-defmethod project-run-target ((obj project-am-objectcode))
   "Run the current project target in comint buffer."
   (require 'ede/shell)
   (let ((tb (get-buffer-create " *padt*"))
@@ -413,11 +409,11 @@ Argument COMMAND is the command to use for compiling the target."
 	  (setq default-directory dd)
 	  (setq cmd (read-from-minibuffer
 		     "Run (like this): "
-		     (concat (ede-target-name obj))))
+		     (concat "./" (ede-target-name obj))))
 	  (ede-shell-run-something obj cmd))
       (kill-buffer tb))))
 
-(defmethod project-make-dist ((this project-am-target))
+(cl-defmethod project-make-dist ((this project-am-target))
   "Run the current project in the debugger."
   (require 'compile)
   (if (not project-am-compile-project-command)
@@ -432,12 +428,8 @@ Argument COMMAND is the command to use for compiling the target."
 If a given set of projects has already been loaded, then do nothing
 but return the project for the directory given.
 Optional ROOTPROJ is the root EDE project."
-  (let* ((ede-constructiong t)
-	 (amo (object-assoc (expand-file-name "Makefile.am" directory)
-			    'file ede-projects)))
-    (when (not amo)
-      (setq amo (project-am-load-makefile directory)))
-    amo))
+  ;; Just jump into creating the project from the Makefiles.
+  (project-am-load-makefile directory))
 
 (defun project-am-find-topmost-level (dir)
   "Find the topmost automakefile starting with DIR."
@@ -499,7 +491,7 @@ This is used when subprojects are made in named subdirectories."
 		    :version ver
 		    :mailinglist (or bug "")
 		    :file fn)))
-	(oset ampf :directory (file-name-directory fn))
+	(oset ampf directory (file-name-directory fn))
 	(oset ampf configureoutputfiles cof)
 	(make-local-variable 'ede-object)
 	(setq ede-object ampf)
@@ -508,7 +500,7 @@ This is used when subprojects are made in named subdirectories."
 	ampf))))
 
 ;;; Methods:
-(defmethod project-targets-for-file ((proj project-am-makefile))
+(cl-defmethod project-targets-for-file ((proj project-am-makefile))
   "Return a list of targets the project PROJ."
   (oref proj targets))
 
@@ -540,7 +532,7 @@ DIR is the directory to apply to new targets."
 		     (project-rescan tmp)
 		     (setq ntargets (cons tmp ntargets)))
 		   (makefile-macro-file-list macro))
-	   ;; Non-indirect will have a target whos sources
+	   ;; Non-indirect will have a target whose sources
 	   ;; are actual files, not names of other targets.
 	   (let ((files (makefile-macro-file-list macro)))
 	     (when files
@@ -620,7 +612,7 @@ Strip out duplicates, and recurse on variables."
 	subdirs)
   )
 
-(defmethod project-rescan ((this project-am-makefile) &optional suggestedname)
+(cl-defmethod project-rescan ((this project-am-makefile) &optional suggestedname)
   "Rescan the makefile for all targets and sub targets."
   (project-am-with-makefile-current (file-name-directory (oref this file))
     ;;(message "Scanning %s..." (oref this file))
@@ -663,15 +655,15 @@ Strip out duplicates, and recurse on variables."
       ;; their object still exists!
       ;; FIGURE THIS OUT
       (project-am-expand-subdirlist 'csubprojexpanded csubproj)
-      ;; Ok, now lets look at all our sub-projects.
+      ;; Ok, now let's look at all our sub-projects.
       (mapc (lambda (sp)
 	      (let* ((subdir (file-name-as-directory
 			      (expand-file-name
-			       sp (file-name-directory (oref this :file)))))
+			       sp (file-name-directory (oref this file)))))
 		     (submake (expand-file-name
 			       "Makefile.am"
 			       subdir)))
-		(if (string= submake (oref this :file))
+		(if (string= submake (oref this file))
 		    nil	;; don't recurse.. please!
 		  ;; For each project id found, see if we need to recycle,
 		  ;; and if we do not, then make a new one.  Check the deep
@@ -700,74 +692,75 @@ Strip out duplicates, and recurse on variables."
       )))
 
 
-(defmethod project-rescan ((this project-am-program))
+(cl-defmethod project-rescan ((this project-am-program))
   "Rescan object THIS."
-  (oset this :source (makefile-macro-file-list (project-am-macro this)))
-  (unless (oref this :source)
-    (oset this :source (list (concat (oref this :name) ".c"))))
-  (oset this :ldadd (makefile-macro-file-list
-		     (concat (oref this :name) "_LDADD"))))
+  (oset this source (makefile-macro-file-list (project-am-macro this)))
+  (unless (oref this source)
+    (oset this source (list (concat (oref this name) ".c"))))
+  (oset this ldadd (makefile-macro-file-list
+		    (concat (oref this name) "_LDADD"))))
 
-(defmethod project-rescan ((this project-am-lib))
+(cl-defmethod project-rescan ((this project-am-lib))
   "Rescan object THIS."
-  (oset this :source (makefile-macro-file-list (project-am-macro this)))
-  (unless (oref this :source)
-    (oset this :source (list (concat (file-name-sans-extension (oref this :name)) ".c")))))
+  (oset this source (makefile-macro-file-list (project-am-macro this)))
+  (unless (oref this source)
+    (oset this source (list (concat (file-name-sans-extension
+                                     (oref this name)) ".c")))))
 
-(defmethod project-rescan ((this project-am-texinfo))
+(cl-defmethod project-rescan ((this project-am-texinfo))
   "Rescan object THIS."
-  (oset this :include (makefile-macro-file-list (project-am-macro this))))
+  (oset this include (makefile-macro-file-list (project-am-macro this))))
 
-(defmethod project-rescan ((this project-am-man))
+(cl-defmethod project-rescan ((this project-am-man))
   "Rescan object THIS."
-  (oset this :source (makefile-macro-file-list (project-am-macro this))))
+  (oset this source (makefile-macro-file-list (project-am-macro this))))
 
-(defmethod project-rescan ((this project-am-lisp))
+(cl-defmethod project-rescan ((this project-am-lisp))
   "Rescan the lisp sources."
-  (oset this :source (makefile-macro-file-list (project-am-macro this))))
+  (oset this source (makefile-macro-file-list (project-am-macro this))))
 
-(defmethod project-rescan ((this project-am-header))
+(cl-defmethod project-rescan ((this project-am-header))
   "Rescan the Header sources for object THIS."
-  (oset this :source (makefile-macro-file-list (project-am-macro this))))
+  (oset this source (makefile-macro-file-list (project-am-macro this))))
 
-(defmethod project-rescan ((this project-am-built-src))
+(cl-defmethod project-rescan ((this project-am-built-src))
   "Rescan built sources for object THIS."
-  (oset this :source (makefile-macro-file-list "BUILT_SOURCES")))
+  (oset this source (makefile-macro-file-list "BUILT_SOURCES")))
 
-(defmethod project-rescan ((this project-am-extra-dist))
+(cl-defmethod project-rescan ((this project-am-extra-dist))
   "Rescan object THIS."
-  (oset this :source (makefile-macro-file-list "EXTRA_DIST")))
+  (oset this source (makefile-macro-file-list "EXTRA_DIST")))
 
-(defmethod project-am-macro ((this project-am-objectcode))
-  "Return the default macro to 'edit' for this object type."
-  (concat (subst-char-in-string ?- ?_ (oref this :name)) "_SOURCES"))
+(cl-defmethod project-am-macro ((this project-am-objectcode))
+  "Return the default macro to `edit' for this object type."
+  (concat (subst-char-in-string ?- ?_ (oref this name)) "_SOURCES"))
 
-(defmethod project-am-macro ((this project-am-header-noinst))
-  "Return the default macro to 'edit' for this object."
+(cl-defmethod project-am-macro ((this project-am-header-noinst))
+  "Return the default macro to `edit' for this object."
   "noinst_HEADERS")
 
-(defmethod project-am-macro ((this project-am-header-inst))
-  "Return the default macro to 'edit' for this object."
+(cl-defmethod project-am-macro ((this project-am-header-inst))
+  "Return the default macro to `edit' for this object."
   "include_HEADERS")
 
-(defmethod project-am-macro ((this project-am-header-pkg))
-  "Return the default macro to 'edit' for this object."
+(cl-defmethod project-am-macro ((this project-am-header-pkg))
+  "Return the default macro to `edit' for this object."
   "pkginclude_HEADERS")
 
-(defmethod project-am-macro ((this project-am-header-chk))
-  "Return the default macro to 'edit' for this object."
+(cl-defmethod project-am-macro ((this project-am-header-chk))
+  "Return the default macro to `edit' for this object."
   "check_HEADERS")
 
-(defmethod project-am-macro ((this project-am-texinfo))
-  "Return the default macro to 'edit' for this object type."
-  (concat (file-name-sans-extension (oref this :name)) "_TEXINFOS"))
+(cl-defmethod project-am-macro ((this project-am-texinfo))
+  "Return the default macro to `edit' for this object type."
+  (concat (file-name-sans-extension (oref this name)) "_TEXINFOS"))
 
-(defmethod project-am-macro ((this project-am-man))
-  "Return the default macro to 'edit' for this object type."
-  (oref this :name))
+(cl-defmethod project-am-macro ((this project-am-man))
+  "Return the default macro to `edit' for this object type."
+  (oref this name))
 
-(defmethod project-am-macro ((this project-am-lisp))
-  "Return the default macro to 'edit' for this object."
+(cl-defmethod project-am-macro ((this project-am-lisp))
+  "Return the default macro to `edit' for this object."
   "lisp_LISP")
 
 (defun project-am-buffer-object (amf buffer)
@@ -789,10 +782,10 @@ nil means that this buffer belongs to no-one."
 		sobj (cdr sobj)))
 	obj))))
 
-(defmethod ede-buffer-mine ((this project-am-makefile) buffer)
+(cl-defmethod ede-buffer-mine ((this project-am-makefile) buffer)
   "Return t if object THIS lays claim to the file in BUFFER."
   (let ((efn  (expand-file-name (buffer-file-name buffer))))
-    (or (string= (oref this :file) efn)
+    (or (string= (oref this file) efn)
 	(string-match "/configure\\.ac$" efn)
 	(string-match "/configure\\.in$" efn)
 	(string-match "/configure$" efn)
@@ -804,44 +797,44 @@ nil means that this buffer belongs to no-one."
 	  ans)
 	)))
 
-(defmethod ede-buffer-mine ((this project-am-objectcode) buffer)
+(cl-defmethod ede-buffer-mine ((this project-am-objectcode) buffer)
   "Return t if object THIS lays claim to the file in BUFFER."
-  (member (file-relative-name (buffer-file-name buffer) (oref this :path))
-	  (oref this :source)))
+  (member (file-relative-name (buffer-file-name buffer) (oref this path))
+	  (oref this source)))
 
-(defmethod ede-buffer-mine ((this project-am-texinfo) buffer)
+(cl-defmethod ede-buffer-mine ((this project-am-texinfo) buffer)
   "Return t if object THIS lays claim to the file in BUFFER."
   (let ((bfn (file-relative-name (buffer-file-name buffer)
-				 (oref this :path))))
-    (or (string= (oref this :name)  bfn)
-	(member bfn (oref this :include)))))
+				 (oref this path))))
+    (or (string= (oref this name)  bfn)
+	(member bfn (oref this include)))))
 
-(defmethod ede-buffer-mine ((this project-am-man) buffer)
+(cl-defmethod ede-buffer-mine ((this project-am-man) buffer)
   "Return t if object THIS lays claim to the file in BUFFER."
-  (string= (oref this :name)
-	   (file-relative-name (buffer-file-name buffer) (oref this :path))))
+  (string= (oref this name)
+	   (file-relative-name (buffer-file-name buffer) (oref this path))))
 
-(defmethod ede-buffer-mine ((this project-am-lisp) buffer)
+(cl-defmethod ede-buffer-mine ((this project-am-lisp) buffer)
   "Return t if object THIS lays claim to the file in BUFFER."
-  (member (file-relative-name (buffer-file-name buffer) (oref this :path))
-	  (oref this :source)))
+  (member (file-relative-name (buffer-file-name buffer) (oref this path))
+	  (oref this source)))
 
-(defmethod project-am-subtree ((ampf project-am-makefile) subdir)
+(cl-defmethod project-am-subtree ((ampf project-am-makefile) subdir)
   "Return the sub project in AMPF specified by SUBDIR."
   (object-assoc (expand-file-name subdir) 'file (oref ampf subproj)))
 
-(defmethod project-compile-target-command ((this project-am-target))
+(cl-defmethod project-compile-target-command ((this project-am-target))
   "Default target to use when compiling a given target."
   ;; This is a pretty good default for most.
   "")
 
-(defmethod project-compile-target-command ((this project-am-objectcode))
+(cl-defmethod project-compile-target-command ((this project-am-objectcode))
   "Default target to use when compiling an object code target."
-  (oref this :name))
+  (oref this name))
 
-(defmethod project-compile-target-command ((this project-am-texinfo))
-  "Default target t- use when compling a texinfo file."
-  (let ((n (oref this :name)))
+(cl-defmethod project-compile-target-command ((this project-am-texinfo))
+  "Default target t- use when compiling a texinfo file."
+  (let ((n (oref this name)))
     (if (string-match "\\.texi?\\(nfo\\)?" n)
 	(setq n (replace-match ".info" t t n)))
     n))
@@ -861,17 +854,17 @@ Argument FILE is the file to extract the end directory name from."
 (defun project-am-preferred-target-type (file)
   "For FILE, return the preferred type for that file."
   (cond ((string-match "\\.texi?\\(nfo\\)$" file)
-	 project-am-texinfo)
+	 'project-am-texinfo)
 	((string-match "\\.[0-9]$" file)
-	 project-am-man)
+	 'project-am-man)
 	((string-match "\\.el$" file)
-	 project-am-lisp)
+	 'project-am-lisp)
 	(t
-	 project-am-program)))
+	 'project-am-program)))
 
-(defmethod ede-buffer-header-file((this project-am-objectcode) buffer)
+(cl-defmethod ede-buffer-header-file((this project-am-objectcode) buffer)
   "There are no default header files."
-  (or (call-next-method)
+  (or (cl-call-next-method)
       (let ((s (oref this source))
 	    (found nil))
 	(while (and s (not found))
@@ -881,7 +874,7 @@ Argument FILE is the file to extract the end directory name from."
 	  (setq s (cdr s)))
 	found)))
 
-(defmethod ede-documentation ((this project-am-texinfo))
+(cl-defmethod ede-documentation ((this project-am-texinfo))
   "Return a list of files that provides documentation.
 Documentation is not for object THIS, but is provided by THIS for other
 files in the project."
@@ -900,10 +893,10 @@ files in the project."
     out))
 
 
-;;; Configure.in queries.
+;;; Configure.ac queries.
 ;;
 (defvar project-am-autoconf-file-options
-  '("configure.in" "configure.ac")
+  '("configure.ac" "configure.in")
   "List of possible configure files to look in for project info.")
 
 (defun project-am-autoconf-file (dir)
@@ -952,7 +945,7 @@ Kill the Configure buffer if it was not already in a buffer."
 	(configfiles nil)
 	)
     (cond
-     ;; Try configure.in or configure.ac
+     ;; Try configure.ac or configure.in
      (conf-in
       (project-am-with-config-current conf-in
 	(let ((aci (autoconf-parameters-for-macro "AC_INIT"))
@@ -1001,16 +994,16 @@ Kill the Configure buffer if it was not already in a buffer."
   "Get the package information for directory topmost project dir over DIR.
 Calculates the info with `project-am-extract-package-info'."
   (let ((top (ede-toplevel)))
-    (when top (setq dir (oref top :directory)))
+    (when top (setq dir (oref top directory)))
     (project-am-extract-package-info dir)))
 
 ;; for simple per project include path extension
-(defmethod ede-system-include-path ((this project-am-makefile))
+(cl-defmethod ede-system-include-path ((this project-am-makefile))
   "Return `project-am-localvars-include-path', usually local variable
 per file or in .dir-locals.el or similar."
   (bound-and-true-p project-am-localvars-include-path))
 
-(defmethod ede-system-include-path ((this project-am-target))
+(cl-defmethod ede-system-include-path ((this project-am-target))
   "Return `project-am-localvars-include-path', usually local variable
 per file or in .dir-locals.el or similar."
   (bound-and-true-p project-am-localvars-include-path))
@@ -1018,5 +1011,8 @@ per file or in .dir-locals.el or similar."
 
 (provide 'ede/project-am)
 
-;; arch-tag: 528db935-f186-4240-b647-e305c5b784a2
+;; Local variables:
+;; generated-autoload-load-name: "ede/project-am"
+;; End:
+
 ;;; ede/project-am.el ends here
