@@ -1,12 +1,12 @@
 ;;; erc-dcc.el --- CTCP DCC module for ERC
 
-;; Copyright (C) 1993-1995, 1998, 2002-2004, 2006-2014 Free Software
+;; Copyright (C) 1993-1995, 1998, 2002-2004, 2006-2021 Free Software
 ;; Foundation, Inc.
 
 ;; Author: Ben A. Mesander <ben@gnu.ai.mit.edu>
 ;;         Noah Friedman <friedman@prep.ai.mit.edu>
 ;;         Per Persson <pp@sno.pp.se>
-;; Maintainer: emacs-devel@gnu.org
+;; Maintainer: Amin Bandali <bandali@gnu.org>
 ;; Keywords: comm, processes
 ;; Created: 1994-01-23
 
@@ -23,7 +23,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -47,16 +47,15 @@
 ;;  /dcc get nick [file] - Accept DCC offer from nick
 ;;  /dcc list - List all DCC offers/connections
 ;;  /dcc send nick file - Offer DCC SEND to nick
-;;
-;; Please note that offering DCC connections (offering chats and sending
-;; files) is only supported with Emacs 22.
 
 ;;; Code:
 
 (require 'erc)
-(eval-when-compile (require 'pcomplete))
+;; Strictly speaking, should only be needed at compile time.
+;; Require at run-time too to silence compiler.
+(require 'pcomplete)
 
-;;;###autoload (autoload 'erc-dcc-mode "erc-dcc")
+;;;###autoload(autoload 'erc-dcc-mode "erc-dcc")
 (define-erc-module dcc nil
   "Provide Direct Client-to-Client support for ERC."
   ((add-hook 'erc-server-401-functions 'erc-dcc-no-such-nick))
@@ -89,13 +88,13 @@ All values of the list must be uppercase strings.")
    (:nick \"nick\" :type SEND :peer server-proc :parent parent-proc :file
    file :sent <marker> :confirmed <marker>))
 
- :nick - a user or userhost for the peer. combine with :parent to reach them
+ :nick - a user or userhost for the peer.  Combine with :parent to reach them
 
  :type - the type of DCC connection - SEND for outgoing files, GET for
-         incoming, and CHAT for both directions. To tell which end started
+         incoming, and CHAT for both directions.  To tell which end started
          the DCC chat, look at :peer
 
- :peer - the other end of the DCC connection. In the case of outgoing DCCs,
+ :peer - the other end of the DCC connection.  In the case of outgoing DCCs,
          this represents a server process until a connection is established
 
  :parent - the server process where the dcc connection was established.
@@ -103,7 +102,7 @@ All values of the list must be uppercase strings.")
            connection is in general independent from a particular server
            connection after it was established.
 
- :file - for outgoing sends, the full path to the file. for incoming sends,
+ :file - for outgoing sends, the full path to the file.  For incoming sends,
          the suggested filename or vetted filename
 
  :size - size of the file, may be nil on incoming DCCs")
@@ -164,11 +163,11 @@ All values of the list must be uppercase strings.")
 
 (defun erc-dcc-member (&rest args)
   "Return the first matching entry in `erc-dcc-list' which satisfies the
-constraints given as a plist in ARGS. Returns nil on no match.
+constraints given as a plist in ARGS.  Returns nil on no match.
 
-The property :nick is treated specially, if it contains a '!' character,
+The property :nick is treated specially, if it contains a `!' character,
 it is treated as a nick!user@host string, and compared with the :nick property
-value of the individual elements using string-equal. Otherwise it is
+value of the individual elements using string-equal.  Otherwise it is
 compared with `erc-nick-equal-p' which is IRC case-insensitive."
   (let ((list erc-dcc-list)
         result test)
@@ -222,14 +221,6 @@ which is big-endian."
       (setq i (1- i)))
     str))
 
-(defconst erc-most-positive-int-bytes
-  (ceiling (/ (ceiling (/ (log most-positive-fixnum) (log 2))) 8.0))
-  "Maximum number of bytes for a fixnum.")
-
-(defconst erc-most-positive-int-msb
-  (lsh most-positive-fixnum (- 0 (* 8 (1- erc-most-positive-int-bytes))))
-  "Content of the most significant byte of most-positive-fixnum.")
-
 (defun erc-unpack-int (str)
   "Unpack a packed string into an integer."
   (let ((len (length str)))
@@ -240,16 +231,11 @@ which is big-endian."
       (when (> start 0)
         (setq str (substring str start))
         (setq len (- len start))))
-    ;; make sure size is not larger than Emacs can handle
-    (when (or (> len (min 4 erc-most-positive-int-bytes))
-              (and (eq len erc-most-positive-int-bytes)
-                   (> (aref str 0) erc-most-positive-int-msb)))
-      (error "ERC-DCC (erc-unpack-int): packet to send is too large"))
     ;; unpack
     (let ((num 0)
           (count 0))
       (while (< count len)
-        (setq num (+ num (lsh (aref str (- len count 1)) (* 8 count))))
+        (setq num (+ num (ash (aref str (- len count 1)) (* 8 count))))
         (setq count (1+ count)))
       num)))
 
@@ -315,15 +301,15 @@ Should be set to a string or nil.  If nil, use the value of
 
 (defcustom erc-dcc-send-request 'ask
   "How to treat incoming DCC Send requests.
-'ask - Report the Send request, and wait for the user to manually accept it
-       You might want to set `erc-dcc-auto-masks' for this.
-'auto - Automatically accept the request and begin downloading the file
-'ignore - Ignore incoming DCC Send requests completely."
+`ask' - Report the Send request, and wait for the user to manually accept it
+        You might want to set `erc-dcc-auto-masks' for this.
+`auto' - Automatically accept the request and begin downloading the file
+`ignore' - Ignore incoming DCC Send requests completely."
   :group 'erc-dcc
   :type '(choice (const ask) (const auto) (const ignore)))
 
 (defun erc-dcc-get-host (proc)
-  "Returns the local IP address used for an open PROCess."
+  "Return the local IP address used for an open process PROC."
   (format-network-address (process-contact proc :local) t))
 
 (defun erc-dcc-host ()
@@ -352,8 +338,8 @@ string \".*!.*@.*\" to this list."
   :type '(repeat regexp))
 
 (defun erc-dcc-server (name filter sentinel)
-  "Start listening on a port for an incoming DCC connection. Returns the newly
-created subprocess, or nil."
+  "Start listening on a port for an incoming DCC connection.
+Returns the newly created subprocess, or nil."
   (let ((port (or (and erc-dcc-port-range (car erc-dcc-port-range)) t))
         (upper (and erc-dcc-port-range (cdr erc-dcc-port-range)))
         process)
@@ -365,7 +351,6 @@ created subprocess, or nil."
                                         :buffer nil
                                         :host (erc-dcc-host)
                                         :service port
-                                        :nowait t
                                         :noquery nil
                                         :filter filter
                                         :sentinel sentinel
@@ -379,7 +364,7 @@ created subprocess, or nil."
                   (set-process-filter-multibyte process nil)))))
         (file-error
          (unless (and (string= "Cannot bind server socket" (nth 1 err))
-                      (string= "address already in use" (nth 2 err)))
+                      (string= "address already in use" (downcase (nth 2 err))))
            (signal (car err) (cdr err)))
          (setq port (1+ port))
          (unless (< port upper)
@@ -428,28 +413,28 @@ where FOO is one of CLOSE, GET, SEND, LIST, CHAT, etc."
 
 ;;;###autoload
 (defun pcomplete/erc-mode/DCC ()
-  "Provides completion for the /DCC command."
+  "Provide completion for the /DCC command."
   (pcomplete-here (append '("chat" "close" "get" "list")
                           (when (fboundp 'make-network-process) '("send"))))
   (pcomplete-here
    (pcase (intern (downcase (pcomplete-arg 1)))
-     (`chat (mapcar (lambda (elt) (plist-get elt :nick))
+     ('chat (mapcar (lambda (elt) (plist-get elt :nick))
                     (erc-remove-if-not
                      #'(lambda (elt)
                          (eq (plist-get elt :type) 'CHAT))
                      erc-dcc-list)))
-     (`close (erc-delete-dups
+     ('close (erc-delete-dups
               (mapcar (lambda (elt) (symbol-name (plist-get elt :type)))
                       erc-dcc-list)))
-     (`get (mapcar #'erc-dcc-nick
+     ('get (mapcar #'erc-dcc-nick
                    (erc-remove-if-not
                     #'(lambda (elt)
                         (eq (plist-get elt :type) 'GET))
                     erc-dcc-list)))
-     (`send (pcomplete-erc-all-nicks))))
+     ('send (pcomplete-erc-all-nicks))))
   (pcomplete-here
    (pcase (intern (downcase (pcomplete-arg 2)))
-     (`get (mapcar (lambda (elt) (plist-get elt :file))
+     ('get (mapcar (lambda (elt) (plist-get elt :file))
                    (erc-remove-if-not
                     #'(lambda (elt)
                         (and (eq (plist-get elt :type) 'GET)
@@ -457,13 +442,13 @@ where FOO is one of CLOSE, GET, SEND, LIST, CHAT, etc."
                                                 (plist-get elt :nick))
                                                (pcomplete-arg 1))))
                     erc-dcc-list)))
-     (`close (mapcar #'erc-dcc-nick
+     ('close (mapcar #'erc-dcc-nick
                      (erc-remove-if-not
                       #'(lambda (elt)
                           (eq (plist-get elt :type)
                               (intern (upcase (pcomplete-arg 1)))))
                       erc-dcc-list)))
-     (`send (pcomplete-entries)))))
+     ('send (pcomplete-entries)))))
 
 (defun erc-dcc-do-CHAT-command (proc &optional nick)
   (when nick
@@ -594,14 +579,9 @@ It lists the current state of `erc-dcc-list' in an easy to read manner."
                                             (get-buffer (plist-get elt :file))
                                           (+ (buffer-size) 0.0
                                              erc-dcc-byte-count))))
-                        (concat " ("
-                                (if (= byte-count 0)
-                                    "0"
-                                  (number-to-string
-                                   (truncate
-                                    (* 100
-                                       (/ byte-count (plist-get elt :size))))))
-                                "%)"))))
+                        (format " (%d%%)"
+                                (floor (* 100.0 byte-count)
+                                       (plist-get elt :size))))))
        ?f (or (and (plist-member elt :file) (plist-get elt :file)) "")))
     (erc-display-message
      nil 'notice 'active
@@ -654,9 +634,10 @@ that subcommand."
           "\"\\(\\(.*?\\(\\\\\"\\)?\\)+?\\)\"\\|\\([^ ]+\\)"
           "\\) \\([0-9]+\\) \\([0-9]+\\) *\\([0-9]*\\)"))
 
-(defsubst erc-dcc-unquote-filename (filename)
-  (erc-replace-regexp-in-string "\\\\\\\\" "\\"
-                                (erc-replace-regexp-in-string "\\\\\"" "\"" filename t t) t t))
+(define-inline erc-dcc-unquote-filename (filename)
+  (inline-quote
+   (erc-replace-regexp-in-string "\\\\\\\\" "\\"
+                                 (erc-replace-regexp-in-string "\\\\\"" "\"" ,filename t t) t t)))
 
 (defun erc-dcc-handle-ctcp-send (proc query nick login host to)
   "This is called if a CTCP DCC SEND subcommand is sent to the client.
@@ -704,9 +685,10 @@ It extracts the information about the dcc request and adds it to
        ?n nick ?u login ?h host ?q query)))))
 
 (defun erc-dcc-auto-mask-p (spec)
-  "Takes a full SPEC of a user in the form \"nick!login@host\" and
-matches against all the regexp's in `erc-dcc-auto-masks'. If any
-match, returns that regexp and nil otherwise."
+  "Match SPEC against `erc-dcc-auto-masks'.
+SPEC is a full spec of a user in the form \"nick!login@host\", which
+is matched against all the regexps in `erc-dcc-auto-masks'.  Return
+the matching regexp, or nil if none found."
   (let ((lst erc-dcc-auto-masks))
     (while (and lst
                 (not (string-match (car lst) spec)))
@@ -718,9 +700,9 @@ match, returns that regexp and nil otherwise."
 
 (defcustom erc-dcc-chat-request 'ask
   "How to treat incoming DCC Chat requests.
-'ask - Report the Chat request, and wait for the user to manually accept it
-'auto - Automatically accept the request and open a new chat window
-'ignore - Ignore incoming DCC chat requests completely."
+`ask' - Report the Chat request, and wait for the user to manually accept it
+`auto' - Automatically accept the request and open a new chat window
+`ignore' - Ignore incoming DCC chat requests completely."
   :group 'erc-dcc
   :type '(choice (const ask) (const auto) (const ignore)))
 
@@ -785,8 +767,8 @@ unconfirmed."
   :group 'erc-dcc
   :type '(choice (const nil) integer))
 
-(defsubst erc-dcc-get-parent (proc)
-  (plist-get (erc-dcc-member :peer proc) :parent))
+(define-inline erc-dcc-get-parent (proc)
+  (inline-quote (plist-get (erc-dcc-member :peer ,proc) :parent)))
 
 (defun erc-dcc-send-block (proc)
   "Send one block of data.
@@ -829,8 +811,8 @@ bytes sent."
          ?s (number-to-string (- sent-marker (point-min))))
         (setq erc-dcc-list (delete elt erc-dcc-list))
         (set-buffer-modified-p nil)
-        (kill-buffer (current-buffer))
-        (delete-process proc))
+        (delete-process proc)
+        (kill-buffer (current-buffer)))
        ((<= confirmed-marker sent-marker)
         (while (and (< (- sent-marker confirmed-marker)
                        (or erc-dcc-pump-bytes
@@ -843,8 +825,8 @@ bytes sent."
                  (marker-position confirmed-marker)
                  (marker-position sent-marker)))
         (set-buffer-modified-p nil)
-        (kill-buffer (current-buffer))
-        (delete-process proc))))))
+        (delete-process proc)
+        (kill-buffer (current-buffer)))))))
 
 (defun erc-dcc-display-send (proc)
   (erc-display-message
@@ -926,9 +908,9 @@ other client."
 (make-variable-buffer-local 'erc-dcc-file-name)
 
 (defun erc-dcc-get-file (entry file parent-proc)
-  "This function does the work of setting up a transfer from the remote client
-to the local one over a tcp connection. This involves setting up a process
-filter and a process sentinel, and making the connection."
+  "Set up a transfer from the remote client to the local over a TCP connection.
+This involves setting up a process filter and a process sentinel,
+and making the connection."
   (let* ((buffer (generate-new-buffer (file-name-nondirectory file)))
          proc)
     (with-current-buffer buffer
@@ -936,10 +918,7 @@ filter and a process sentinel, and making the connection."
       (buffer-disable-undo (current-buffer))
       ;; This is necessary to have the buffer saved as-is in GNU
       ;; Emacs.
-      ;; XEmacs change: We don't have `set-buffer-multibyte', setting
-      ;; coding system to 'binary below takes care of us.
-      (when (fboundp 'set-buffer-multibyte)
-        (set-buffer-multibyte nil))
+      (set-buffer-multibyte nil)
 
       (setq mode-line-process '(":%s")
             buffer-read-only t)
@@ -994,25 +973,29 @@ rather than every 1024 byte block, but nobody seems to care."
     (let ((inhibit-read-only t)
           received-bytes)
       (goto-char (point-max))
-      (insert (string-make-unibyte str))
+      (when str
+        (cl-assert (not (multibyte-string-p str)))
+        (insert str))
 
       (when (> (point-max) erc-dcc-receive-cache)
         (erc-dcc-append-contents (current-buffer) erc-dcc-file-name))
-      (setq received-bytes (+ (buffer-size) erc-dcc-byte-count))
+      (setq received-bytes (buffer-size))
+      (if erc-dcc-byte-count
+          (setq received-bytes (+ received-bytes erc-dcc-byte-count)))
 
       (and erc-dcc-verbose
            (erc-display-message
             nil 'notice erc-server-process
             'dcc-get-bytes-received
-            ?f (file-name-nondirectory buffer-file-name)
+            ?f (file-name-nondirectory (buffer-name))
             ?b (number-to-string received-bytes)))
       (cond
        ((and (> (plist-get erc-dcc-entry-data :size) 0)
              (> received-bytes (plist-get erc-dcc-entry-data :size)))
         (erc-display-message
-         nil '(error notice) 'active
+         nil '(notice error) 'active
          'dcc-get-file-too-long
-         ?f (file-name-nondirectory buffer-file-name))
+         ?f (file-name-nondirectory (buffer-name)))
         (delete-process proc))
        (t
         (process-send-string
@@ -1036,7 +1019,7 @@ transfer is complete."
      ?s (number-to-string erc-dcc-byte-count)
      ?t (format "%.0f"
                 (erc-time-diff (plist-get erc-dcc-entry-data :start-time)
-                               (erc-current-time)))))
+                               nil))))
   (kill-buffer (process-buffer proc))
   (delete-process proc))
 
@@ -1096,14 +1079,14 @@ Possible values are: ask, auto, ignore."
   (pcomplete-here '("auto" "ask" "ignore")))
 (defalias 'pcomplete/erc-mode/SREQ 'pcomplete/erc-mode/CREQ)
 
+(define-obsolete-variable-alias 'erc-dcc-chat-filter-hook
+  'erc-dcc-chat-filter-functions "24.3")
+
 (defvar erc-dcc-chat-filter-functions '(erc-dcc-chat-parse-output)
   "Abnormal hook run after parsing (and maybe inserting) a DCC message.
 Each function is called with two arguments: the ERC process and
 the unprocessed output.")
 
-(define-obsolete-variable-alias 'erc-dcc-chat-filter-hook
-  'erc-dcc-chat-filter-functions "24.3")
-
 (defvar erc-dcc-chat-mode-map
   (let ((map (make-sparse-keymap)))
     (define-key map (kbd "RET") 'erc-send-current-line)
@@ -1174,7 +1157,7 @@ other client."
     buffer))
 
 (defun erc-dcc-chat-accept (entry parent-proc)
-  "Accept an incoming DCC connection and open a DCC window"
+  "Accept an incoming DCC connection and open a DCC window."
   (let* ((nick (erc-extract-nick (plist-get entry :nick)))
          buffer proc)
     (setq proc
@@ -1210,7 +1193,7 @@ other client."
         (setq posn (match-end 0))
         (erc-display-message
          nil nil proc
-         'dcc-chat-privmsg ?n (erc-propertize erc-dcc-from 'face
+         'dcc-chat-privmsg ?n (erc-propertize erc-dcc-from 'font-lock-face
                                               'erc-nick-default-face) ?m line))
       (setq erc-dcc-unprocessed-output (substring str posn)))))
 
@@ -1262,6 +1245,5 @@ other client."
 ;;; erc-dcc.el ends here
 ;;
 ;; Local Variables:
-;; indent-tabs-mode: nil
+;; generated-autoload-file: "erc-loaddefs.el"
 ;; End:
-
