@@ -1,11 +1,11 @@
-;;; find-dired.el --- run a `find' command and dired the output
+;;; find-dired.el --- run a `find' command and dired the output  -*- lexical-binding: t -*-
 
-;; Copyright (C) 1992, 1994, 1995, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 1994-1995, 2000-2021 Free Software Foundation,
+;; Inc.
 
 ;; Author: Roland McGrath <roland@gnu.org>,
 ;;	   Sebastian Kremer <sk@thp.uni-koeln.de>
-;; Maintainer: FSF
+;; Maintainer: emacs-devel@gnu.org
 ;; Keywords: unix
 
 ;; This file is part of GNU Emacs.
@@ -21,7 +21,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -30,40 +30,95 @@
 (require 'dired)
 
 (defgroup find-dired nil
-  "Run a `find' command and dired the output."
+  "Run a `find' command and Dired the output."
   :group 'dired
   :prefix "find-")
 
+;; FIXME this option does not really belong in this file, it's more general.
+;; Eg cf some tests in grep.el.
+(defcustom find-exec-terminator
+  (if (eq 0
+	  (ignore-errors
+	    (process-file find-program nil nil nil
+			  null-device "-exec" "echo" "{}" "+")))
+      "+"
+    (shell-quote-argument ";"))
+  "String that terminates \"find -exec COMMAND {} \".
+The value should include any needed quoting for the shell.
+Common values are \"+\" and \"\\\\;\", with the former more efficient
+than the latter."
+  :version "24.1"
+  :group 'find-dired
+  :type 'string)
+
+(defvar find-ls-option-default-ls
+  (cons "-ls" (if (eq system-type 'berkeley-unix) "-gilsb" "-dilsb")))
+
+(defvar find-ls-option-default-exec
+  (cons (format "-exec ls -ld {} %s" find-exec-terminator) "-ld"))
+
+(defvar find-ls-option-default-xargs
+  (cons "-print0 | sort -z | xargs -0 -e ls -ld" "-ld"))
+
 ;; find's -ls corresponds to these switches.
 ;; Note -b, at least GNU find quotes spaces etc. in filenames
-;;;###autoload
 (defcustom find-ls-option
-  (if (eq system-type 'berkeley-unix) (purecopy '("-ls" . "-gilsb"))
-    (purecopy '("-exec ls -ld {} \\;" . "-ld")))
-  "Description of the option to `find' to produce an `ls -l'-type listing.
-This is a cons of two strings (FIND-OPTION . LS-SWITCHES).  FIND-OPTION
-gives the option (or options) to `find' that produce the desired output.
-LS-SWITCHES is a list of `ls' switches to tell dired how to parse the output."
-  :type '(cons (string :tag "Find Option")
-	       (string :tag "Ls Switches"))
+  (if (eq 0
+	  (ignore-errors
+	    (process-file find-program nil nil nil null-device "-ls")))
+      find-ls-option-default-ls
+    find-ls-option-default-exec)
+  "A pair of options to produce and parse an `ls -l'-type list from `find'.
+This is a cons of two strings (FIND-OPTION . LS-SWITCHES).
+FIND-OPTION is the option (or options) passed to `find' to produce
+a file listing in the desired format.  LS-SWITCHES is a set of
+`ls' switches that tell dired how to parse the output of `find'.
+
+The two options must be set to compatible values.
+For example, to use human-readable file sizes with GNU ls:
+   (\"-exec ls -ldh {} +\" . \"-ldh\")
+
+To use GNU find's inbuilt \"-ls\" option to list files:
+   (\"-ls\" . \"-dilsb\")
+since GNU find's output has the same format as using GNU ls with
+the options \"-dilsb\".
+
+While the option `find -ls' often produces unsorted output, the option
+`find -exec ls -ld' maintains the sorting order only on short output,
+whereas `find -print | sort | xargs' produced sorted output even
+on the large number of files."
+  :version "27.1"            ; add choice of predefined set of options
+  :type `(choice
+          (cons :tag "find -ls"
+                (string ,(car find-ls-option-default-ls))
+                (string ,(cdr find-ls-option-default-ls)))
+          (cons :tag "find -exec ls -ld"
+                (string ,(car find-ls-option-default-exec))
+                (string ,(cdr find-ls-option-default-exec)))
+          (cons :tag "find -print | sort | xargs"
+                (string ,(car find-ls-option-default-xargs))
+                (string ,(cdr find-ls-option-default-xargs)))
+          (cons :tag "Other values"
+                (string :tag "Find Option")
+                (string :tag "Ls Switches")))
   :group 'find-dired)
 
-;;;###autoload
-(defcustom find-ls-subdir-switches (purecopy "-al")
+(defcustom find-ls-subdir-switches
+  (if (string-match "-[a-z]*b" (cdr find-ls-option))
+      "-alb"
+    "-al")
   "`ls' switches for inserting subdirectories in `*Find*' buffers.
 This should contain the \"-l\" switch.
 Use the \"-F\" or \"-b\" switches if and only if you also use
 them for `find-ls-option'."
+  :version "24.1"			; add -b test
   :type 'string
-  :group 'find-dired
-  :version "22.1")
+  :group 'find-dired)
 
-;;;###autoload
 (defcustom find-grep-options
-  (purecopy (if (or (eq system-type 'berkeley-unix)
-	  (string-match "solaris2" system-configuration)
-	  (string-match "irix" system-configuration))
-      "-s" "-q"))
+  (if (or (eq system-type 'berkeley-unix)
+	  (string-match "solaris2" system-configuration))
+      "-s" "-q")
   "Option to grep to be as silent as possible.
 On Berkeley systems, this is `-s'; on Posix, and with GNU grep, `-q' does it.
 On other systems, the closest you can come is to use `-l'."
@@ -72,9 +127,9 @@ On other systems, the closest you can come is to use `-l'."
 
 ;; This used to be autoloaded (see bug#4387).
 (defcustom find-name-arg
-  (purecopy (if read-file-name-completion-ignore-case
+  (if read-file-name-completion-ignore-case
       "-iname"
-    "-name"))
+    "-name")
   "Argument used to specify file name pattern.
 If `read-file-name-completion-ignore-case' is non-nil, -iname is used so that
 find also ignores case.  Otherwise, -name is used."
@@ -82,6 +137,17 @@ find also ignores case.  Otherwise, -name is used."
   :group 'find-dired
   :version "22.2")
 
+(defcustom find-dired-refine-function #'find-dired-sort-by-filename
+  "If non-nil, a function for refining the *Find* buffer of `find-dired'.
+This function takes no arguments.  The *Find* buffer is narrowed to the
+output of `find' (one file per line) when this function is called."
+  :version "27.1"
+  :group 'find-dired
+  :type '(choice (const :tag "Sort file names lexicographically"
+                        find-dired-sort-by-filename)
+                 (function :tag "Refining function")
+                 (const :tag "No refining" nil)))
+
 (defvar find-args nil
   "Last arguments given to `find' by \\[find-dired].")
 
@@ -93,13 +159,13 @@ find also ignores case.  Otherwise, -name is used."
 ;;;###autoload
 (defun find-dired (dir args)
   "Run `find' and go into Dired mode on a buffer of the output.
-The command run (after changing into DIR) is
+The command run (after changing into DIR) is essentially
 
     find . \\( ARGS \\) -ls
 
-except that the variable `find-ls-option' specifies what to use
-as the final argument."
-  (interactive (list (read-file-name "Run find in directory: " nil "" t)
+except that the car of the variable `find-ls-option' specifies what to
+use in place of \"-ls\" as the final argument."
+  (interactive (list (read-directory-name "Run find in directory: " nil "" t)
 		     (read-string "Run find (with args): " find-args
 				  '(find-args-history . 1))))
   (let ((dired-buffers dired-buffers))
@@ -109,14 +175,15 @@ as the final argument."
     ;; Check that it's really a directory.
     (or (file-directory-p dir)
 	(error "find-dired needs a directory: %s" dir))
-    (switch-to-buffer (get-buffer-create "*Find*"))
+    (pop-to-buffer-same-window (get-buffer-create "*Find*"))
 
     ;; See if there's still a `find' running, and offer to kill
     ;; it first, if it is.
     (let ((find (get-buffer-process (current-buffer))))
       (when find
 	(if (or (not (eq (process-status find) 'run))
-		(yes-or-no-p "A `find' process is running; kill it? "))
+		(yes-or-no-p
+		 (format-message "A `find' process is running; kill it? ")))
 	    (condition-case nil
 		(progn
 		  (interrupt-process find)
@@ -139,11 +206,12 @@ as the final argument."
 			  " " args " "
 			  (shell-quote-argument ")")
 			  " "))
-		       (if (equal (car find-ls-option) "-exec ls -ld {} \\;")
-			   (concat "-exec ls -ld "
+		       (if (string-match "\\`\\(.*\\) {} \\(\\\\;\\|\\+\\)\\'"
+					 (car find-ls-option))
+			   (format "%s %s %s"
+				   (match-string 1 (car find-ls-option))
 				   (shell-quote-argument "{}")
-				   " "
-				   (shell-quote-argument ";"))
+				   find-exec-terminator)
 			 (car find-ls-option))))
     ;; Start the find process.
     (shell-command (concat args "&") (current-buffer))
@@ -174,13 +242,15 @@ as the final argument."
     (insert "  " dir ":\n")
     ;; Make second line a ``find'' line in analogy to the ``total'' or
     ;; ``wildcard'' line.
-    (insert "  " args "\n")
+    (let ((point (point)))
+      (insert "  " args "\n")
+      (dired-insert-set-properties point (point)))
     (setq buffer-read-only t)
     (let ((proc (get-buffer-process (current-buffer))))
       (set-process-filter proc (function find-dired-filter))
       (set-process-sentinel proc (function find-dired-sentinel))
       ;; Initialize the process marker; it is used by the filter.
-      (move-marker (process-mark proc) 1 (current-buffer)))
+      (move-marker (process-mark proc) (point) (current-buffer)))
     (setq mode-line-process '(":%s"))))
 
 (defun kill-find ()
@@ -196,11 +266,13 @@ as the final argument."
 ;;;###autoload
 (defun find-name-dired (dir pattern)
   "Search DIR recursively for files matching the globbing pattern PATTERN,
-and run dired on those files.
+and run Dired on those files.
 PATTERN is a shell wildcard (not an Emacs regexp) and need not be quoted.
-The command run (after changing into DIR) is
+The default command run (after changing into DIR) is
 
-    find . -name 'PATTERN' -ls"
+    find . -name \\='PATTERN\\=' -ls
+
+See `find-name-arg' to customize the arguments."
   (interactive
    "DFind-name (directory): \nsFind-name (filename wildcard): ")
   (find-dired dir (concat find-name-arg " " (shell-quote-argument pattern))))
@@ -214,12 +286,17 @@ The command run (after changing into DIR) is
 (defalias 'lookfor-dired 'find-grep-dired)
 ;;;###autoload
 (defun find-grep-dired (dir regexp)
-  "Find files in DIR containing a regexp REGEXP and start Dired on output.
+  "Find files in DIR that contain matches for REGEXP and start Dired on output.
 The command run (after changing into DIR) is
 
-    find . -exec grep -s -e REGEXP {} \\\; -ls
+  find . \\( -type f -exec `grep-program' `find-grep-options' \\
+    -e REGEXP {} \\; \\) -ls
 
-Thus ARG can also contain additional grep options."
+where the first string in the value of the variable `find-ls-option'
+specifies what to use in place of \"-ls\" as the final argument."
+  ;; Doc used to say "Thus ARG can also contain additional grep options."
+  ;; i) Presumably ARG == REGEXP?
+  ;; ii) No it can't have options, since it gets shell-quoted.
   (interactive "DFind-grep (directory): \nsFind-grep (grep regexp): ")
   ;; find -exec doesn't allow shell i/o redirections in the command,
   ;; or we could use `grep -l >/dev/null'
@@ -232,6 +309,7 @@ Thus ARG can also contain additional grep options."
 		      " "
 		      (shell-quote-argument "{}")
 		      " "
+		      ;; Doesn't work with "+".
 		      (shell-quote-argument ";"))))
 
 (defun find-dired-filter (proc string)
@@ -248,7 +326,7 @@ Thus ARG can also contain additional grep options."
 		    (l-opt (and (consp find-ls-option)
 				(string-match "l" (cdr find-ls-option))))
 		    (ls-regexp (concat "^ +[^ \t\r\n]+\\( +[^ \t\r\n]+\\) +"
-				       "[^ \t\r\n]+ +[^ \t\r\n]+\\( +[0-9]+\\)")))
+				       "[^ \t\r\n]+ +[^ \t\r\n]+\\( +[^[:space:]]+\\)")))
 		(goto-char beg)
 		(insert string)
 		(goto-char beg)
@@ -287,30 +365,45 @@ Thus ARG can also contain additional grep options."
       (delete-process proc))))
 
 (defun find-dired-sentinel (proc state)
-  ;; Sentinel for \\[find-dired] processes.
-  (let ((buf (process-buffer proc))
-	(inhibit-read-only t))
+  "Sentinel for \\[find-dired] processes."
+  (let ((buf (process-buffer proc)))
     (if (buffer-name buf)
 	(with-current-buffer buf
-	  (let ((buffer-read-only nil))
+	  (let ((inhibit-read-only t))
 	    (save-excursion
-	      (goto-char (point-max))
-	      (insert "\n  find " state)
-	      (forward-char -1)		;Back up before \n at end of STATE.
-	      (insert " at " (substring (current-time-string) 0 19))
-	      (forward-char 1)
-	      (setq mode-line-process
-		    (concat ":"
-			    (symbol-name (process-status proc))))
+              (save-restriction
+                (widen)
+                (when find-dired-refine-function
+                  ;; `find-dired-filter' puts two whitespace characters
+                  ;; at the beginning of every line.
+                  (narrow-to-region (point) (- (point-max) 2))
+                  (funcall find-dired-refine-function)
+                  (widen))
+                (let ((point (point-max)))
+                  (goto-char point)
+                  (insert "\n  find "
+                          (substring state 0 -1) ; omit \n at end of STATE.
+                          " at " (substring (current-time-string) 0 19))
+                  (dired-insert-set-properties point (point))))
+              (setq mode-line-process
+		    (format ":%s" (process-status proc)))
 	      ;; Since the buffer and mode line will show that the
 	      ;; process is dead, we can delete it now.  Otherwise it
-	      ;; will stay around until M-x list-processes.
+	      ;; will stay around until M-x `list-processes'.
 	      (delete-process proc)
-	      (force-mode-line-update)))
-	  (message "find-dired %s finished." (current-buffer))))))
+	      (force-mode-line-update))))
+	  (message "find-dired %s finished." buf))))
+
+(defun find-dired-sort-by-filename ()
+  "Sort entries in *Find* buffer by file name lexicographically."
+  (sort-subr nil 'forward-line 'end-of-line
+             (lambda ()
+               (buffer-substring-no-properties
+                (next-single-property-change
+                 (point) 'dired-filename)
+                (line-end-position)))))
 
 
 (provide 'find-dired)
 
-;; arch-tag: 8edece95-af00-4221-bc74-a4bd2f75f9b0
 ;;; find-dired.el ends here
