@@ -1,6 +1,6 @@
 ;;; mh-tool-bar.el --- MH-E tool bar support
 
-;; Copyright (C) 2002-2003, 2005-2018 Free Software Foundation, Inc.
+;; Copyright (C) 2002-2003, 2005-2021 Free Software Foundation, Inc.
 
 ;; Author: Satyaki Das <satyaki@theforce.stanford.edu>
 ;; Maintainer: Bill Wohler <wohler@newt.com>
@@ -36,7 +36,7 @@
 
 ;;; Tool Bar Commands
 
-(defun mh-tool-bar-search (&optional arg)
+(defun mh-tool-bar-search (&optional _arg)
   "Interactively call `mh-tool-bar-search-function'.
 Optional argument ARG is not used."
   (interactive "P")
@@ -131,11 +131,12 @@ where,
   active. If it isn't present then the button is always active."
   ;; The following variable names have been carefully chosen to make code
   ;; generation easier. Modifying the names should be done carefully.
-  (let (folder-buttons folder-docs folder-button-setter sequence-button-setter
-                       show-buttons show-button-setter show-seq-button-setter
-                       letter-buttons letter-docs letter-button-setter
-                       folder-defaults letter-defaults
-                       folder-vectors show-vectors letter-vectors)
+  (mh-dlet* (folder-buttons
+             folder-docs folder-button-setter sequence-button-setter
+             show-buttons show-button-setter show-seq-button-setter
+             letter-buttons letter-docs letter-button-setter
+             folder-defaults letter-defaults
+             folder-vectors show-vectors letter-vectors)
     (dolist (x defaults)
       (cond ((eq (car x) :folder) (setq folder-defaults (cdr x)))
             ((eq (car x) :letter) (setq letter-defaults (cdr x)))))
@@ -161,14 +162,14 @@ where,
                 (append `(,(if (memq 'folder modes) :folder :sequence) ,name)
                         functions))
           (setq show-sym
-                (if (string-match "^mh-\\(.*\\)$" name-str)
+                (if (string-match "\\`mh-\\(.*\\)\\'" name-str)
                     (intern (concat "mh-show-" (match-string 1 name-str)))
                   name))
           (setq functions
                 (append `(,(if (memq 'folder modes) :show :show-seq)
                           ,(if (fboundp show-sym) show-sym name))
                         functions)))
-        (do ((functions functions (cddr functions)))
+        (cl-do ((functions functions (cddr functions)))
             ((null functions))
           (let* ((type (car functions))
                  (function (cadr functions))
@@ -209,15 +210,15 @@ where,
     (dolist (x letter-defaults)
       (unless (memq x letter-buttons)
         (error "Letter defaults contains unknown button %s" x)))
-    `(eval-when (compile load eval)
+    `(eval-and-compile
        ;; GNU Emacs tool bar specific code
        (mh-do-in-gnu-emacs
          (defun mh-buffer-exists-p (mode)
            "Test whether a buffer with major mode MODE is present."
-           (loop for buf in (buffer-list)
-                 when (with-current-buffer buf
-                        (eq major-mode mode))
-                 return t))
+           (cl-loop for buf in (buffer-list)
+                    when (with-current-buffer buf
+                           (eq major-mode mode))
+                    return t))
          ;; Tool bar initialization functions
          (defun mh-tool-bar-folder-buttons-init ()
            (when (mh-buffer-exists-p 'mh-folder-mode)
@@ -257,18 +258,18 @@ where,
          (defun mh-tool-bar-update (mode default-map sequence-map)
            "Update `tool-bar-map' in all buffers of MODE.
 Use SEQUENCE-MAP if display is limited; DEFAULT-MAP otherwise."
-           (loop for buf in (buffer-list)
-                 do (with-current-buffer buf
-                      (if (eq mode major-mode)
-                          (let ((map (if mh-folder-view-stack
-                                         sequence-map
-                                       default-map)))
-                            ;; Yes, make-local-variable is necessary since we
-                            ;; get here during initialization when loading
-                            ;; mh-e.el, after the +inbox buffer has been
-                            ;; created, but before mh-folder-mode has run and
-                            ;; created the local map.
-                            (set (make-local-variable 'tool-bar-map) map))))))
+           (cl-loop for buf in (buffer-list)
+                    do (with-current-buffer buf
+                         (when (eq mode major-mode) ;FIXME: derived-mode-p?
+                           (let ((map (if mh-folder-view-stack
+                                          sequence-map
+                                        default-map)))
+                             ;; Yes, make-local-variable is necessary since we
+                             ;; get here during initialization when loading
+                             ;; mh-e.el, after the +inbox buffer has been
+                             ;; created, but before mh-folder-mode has run and
+                             ;; created the local map.
+                             (set (make-local-variable 'tool-bar-map) map))))))
          (defun mh-tool-bar-folder-buttons-set (symbol value)
            "Construct tool bar for `mh-folder-mode' and `mh-show-mode'."
            (set-default symbol value)
@@ -286,17 +287,17 @@ Use SEQUENCE-MAP if display is limited; DEFAULT-MAP otherwise."
        ;; XEmacs specific code
        (mh-do-in-xemacs
          (defvar mh-tool-bar-folder-vector-map
-           (list ,@(loop for button in folder-buttons
-                     for vector in folder-vectors
-                     collect `(cons ',button ,vector))))
+           (list ,@(cl-loop for button in folder-buttons
+                            for vector in folder-vectors
+                            collect `(cons ',button ,vector))))
          (defvar mh-tool-bar-show-vector-map
-           (list ,@(loop for button in show-buttons
-                     for vector in show-vectors
-                     collect `(cons ',button ,vector))))
+           (list ,@(cl-loop for button in show-buttons
+                            for vector in show-vectors
+                            collect `(cons ',button ,vector))))
          (defvar mh-tool-bar-letter-vector-map
-           (list ,@(loop for button in letter-buttons
-                     for vector in letter-vectors
-                     collect `(cons ',button ,vector))))
+           (list ,@(cl-loop for button in letter-buttons
+                            for vector in letter-vectors
+                            collect `(cons ',button ,vector))))
          (defvar mh-tool-bar-folder-buttons)
          (defvar mh-tool-bar-show-buttons)
          (defvar mh-tool-bar-letter-buttons)
@@ -305,18 +306,20 @@ Use SEQUENCE-MAP if display is limited; DEFAULT-MAP otherwise."
            (set-default symbol value)
            (when mh-xemacs-has-tool-bar-flag
              (setq mh-tool-bar-letter-buttons
-                   (loop for b in value
-                         collect (cdr
-                                  (assoc b mh-tool-bar-letter-vector-map))))))
+                   (cl-loop
+                    for b in value
+                    collect (cdr (assoc b mh-tool-bar-letter-vector-map))))))
          (defun mh-tool-bar-folder-buttons-set (symbol value)
            (set-default symbol value)
            (when mh-xemacs-has-tool-bar-flag
              (setq mh-tool-bar-folder-buttons
-                   (loop for b in value
-                         collect (cdr (assoc b mh-tool-bar-folder-vector-map))))
+                   (cl-loop
+                    for b in value
+                    collect (cdr (assoc b mh-tool-bar-folder-vector-map))))
              (setq mh-tool-bar-show-buttons
-                   (loop for b in value
-                         collect (cdr (assoc b mh-tool-bar-show-vector-map))))))
+                   (cl-loop
+                    for b in value
+                    collect (cdr (assoc b mh-tool-bar-show-vector-map))))))
          (defun mh-tool-bar-init (mode)
            "Install tool bar in MODE."
            (when mh-xemacs-use-tool-bar-flag
@@ -354,9 +357,9 @@ Use SEQUENCE-MAP if display is limited; DEFAULT-MAP otherwise."
         "List of buttons to include in MH-Folder tool bar."
         :group 'mh-tool-bar
         :set 'mh-tool-bar-folder-buttons-set
-        :type '(set ,@(loop for x in folder-buttons
-                            for y in folder-docs
-                            collect `(const :tag ,y ,x)))
+        :type '(set ,@(cl-loop for x in folder-buttons
+                               for y in folder-docs
+                               collect `(const :tag ,y ,x)))
         ;;:package-version '(MH-E "7.1")
         )
        (custom-declare-variable
@@ -365,9 +368,9 @@ Use SEQUENCE-MAP if display is limited; DEFAULT-MAP otherwise."
         "List of buttons to include in MH-Letter tool bar."
         :group 'mh-tool-bar
         :set 'mh-tool-bar-letter-buttons-set
-        :type '(set ,@(loop for x in letter-buttons
-                            for y in letter-docs
-                            collect `(const :tag ,y ,x)))
+        :type '(set ,@(cl-loop for x in letter-buttons
+                               for y in letter-docs
+                               collect `(const :tag ,y ,x)))
         ;;:package-version '(MH-E "7.1")
         ))))
 
