@@ -1,9 +1,9 @@
 ;;; texnfo-upd.el --- utilities for updating nodes and menus in Texinfo files
 
-;; Copyright (C) 1989-1992, 2001-2015 Free Software Foundation, Inc.
+;; Copyright (C) 1989-1992, 2001-2021 Free Software Foundation, Inc.
 
 ;; Author: Robert J. Chassell
-;; Maintainer: bug-texinfo@gnu.org
+;; Maintainer: emacs-devel@gnu.org
 ;; Keywords: maint, tex, docs
 
 ;; This file is part of GNU Emacs.
@@ -19,7 +19,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -410,23 +410,24 @@ and to the end of the menu region for the level.
 Return t if the node is found, else nil.  Leave point at the beginning
 of the node if one is found; else do not move point."
   (let ((case-fold-search t))
-    (if (and (< (point) region-end)
-	     (re-search-forward
-	      (concat
-	       "\\(^@node\\).*\n"         ; match node line
-	       "\\(\\(\\(^@c\\).*\n\\)"   ; match comment line, if any
-	       "\\|"                      ; or
-	       "\\(^@ifinfo[ ]*\n\\)"     ; ifinfo line, if any
-               "\\|"                      ; or
-               "\\(^@ifnottex[ ]*\n\\)"   ; ifnottex line, if any
-               "\\)?"                     ; end of expression
-	       (eval (cdr (assoc level texinfo-update-menu-lower-regexps))))
-	      ;; the next higher level node marks the end of this
-	      ;; section, and no lower level node will be found beyond
-	      ;; this position even if region-end is farther off
-	      (texinfo-update-menu-region-end level)
-	      t))
-	(goto-char (match-beginning 1)))))
+    (when (and (< (point) region-end)
+	       (re-search-forward
+		(concat
+		 "\\(^@node\\).*\n"	    ; match node line
+		 "\\(\\(\\(^@c\\).*\n\\)"   ; match comment line, if any
+		 "\\|"			    ; or
+		 "\\(^@ifinfo[ ]*\n\\)"	    ; ifinfo line, if any
+		 "\\|"			    ; or
+		 "\\(^@ifnottex[ ]*\n\\)"   ; ifnottex line, if any
+		 "\\)?"			    ; end of expression
+		 (eval (cdr (assoc level texinfo-update-menu-lower-regexps))))
+		;; the next higher level node marks the end of this
+		;; section, and no lower level node will be found beyond
+		;; this position even if region-end is farther off
+		(texinfo-update-menu-region-end level)
+		t))
+      (goto-char (match-beginning 1))
+      t)))
 
 (defun texinfo-find-higher-level-node (level region-end)
   "Search forward from point for node at any higher level than argument LEVEL.
@@ -519,7 +520,7 @@ line.  If there is no node name, returns an empty string."
 
   (save-excursion
     (buffer-substring
-     (progn (forward-word 1)              ; skip over node command
+     (progn (forward-word-strictly 1)     ; skip over node command
 	    (skip-chars-forward " \t")    ; and over spaces
 	    (point))
      (if (search-forward "," (line-end-position) t) ; bound search
@@ -542,7 +543,7 @@ must have been done by `texinfo-menu-locate-entry-p'."
   (goto-char (match-beginning 7))       ; match section name
 
   (buffer-substring
-   (progn (forward-word 1)              ; skip over section type
+   (progn (forward-word-strictly 1)     ; skip over section type
 	  (skip-chars-forward " \t")    ; and over spaces
 	  (point))
    (progn (end-of-line) (point))))
@@ -642,7 +643,7 @@ appears in the texinfo file."
   "Return description field of old menu line as string.
 Point must be located just after the node name.  Point left before description.
 Single argument, END-OF-MENU, is position limiting search."
-  (skip-chars-forward "[:.,\t\n ]+")
+  (skip-chars-forward ":.,\t\n ")
   ;; don't copy a carriage return at line beginning with asterisk!
   ;; don't copy @detailmenu or @end menu or @ignore as descriptions!
   ;; do copy a description that begins with an `@'!
@@ -694,8 +695,8 @@ section titles are often too short to explain a node well.
 
 MENU-LIST has form:
 
-    \(\(\"node-name1\" . \"description\"\)
-    \(\"node-name2\" . \"description\"\) ... \)
+    ((\"node-name1\" . \"description\")
+     (\"node-name2\" . \"description\") ... )
 
 However, the description field might be nil.
 
@@ -794,7 +795,7 @@ complements the node name rather than repeats it as a title does."
           (setq title
 		(buffer-substring
 		 ;; skip over section type
-		 (progn (forward-word 1)
+		 (progn (forward-word-strictly 1)
 			;; and over spaces
 			(skip-chars-forward " \t")
 			(point))
@@ -893,7 +894,7 @@ first update all existing menus in the buffer (incorporating
 descriptions from pre-existing menus) before it constructs the
 master menu.  If the argument is numeric (e.g., \"C-u 2\"),
 update all existing nodes as well, by calling
-\`texinfo-update-node' on the entire file.  Warning: do NOT
+`texinfo-update-node' on the entire file.  Warning: do NOT
 invoke with a numeric argument if your Texinfo file uses @node
 lines without the `Next', `Previous', `Up' pointers, as the
 result could be an invalid Texinfo file!
@@ -1002,9 +1003,9 @@ following menu and the title of the node preceding that menu.
 
 The master menu list has this form:
 
-    \(\(\(... \"entry-1-2\"  \"entry-1\"\) \"title-1\"\)
-      \(\(... \"entry-2-2\"  \"entry-2-1\"\) \"title-2\"\)
-      ...\)
+    (((... \"entry-1-2\"  \"entry-1\") \"title-1\")
+     ((... \"entry-2-2\"  \"entry-2-1\") \"title-2\")
+     ...)
 
 However, there does not need to be a title field."
 
@@ -1018,7 +1019,7 @@ However, there does not need to be a title field."
   "Format and insert the master menu in the current buffer."
   (goto-char (point-min))
   ;; Insert a master menu only after `Top' node and before next node
-  ;; \(or include file if there is no next node\).
+  ;; (or include file if there is no next node).
   (unless (re-search-forward "^@node [ \t]*top[ \t]*\\(,\\|$\\)" nil t)
     (error "This buffer needs a Top node"))
   (let ((first-chapter
@@ -1104,7 +1105,7 @@ point."
 	   t)
 	  (progn
 	    (beginning-of-line)
-	    (forward-word 1)              ; skip over section type
+	    (forward-word-strictly 1)     ; skip over section type
 	    (skip-chars-forward " \t")    ; and over spaces
 	    (buffer-substring
 	     (point)
@@ -1145,24 +1146,40 @@ For example, \"unnumberedsubsec\".  Return \"top\" for top node.
 Searches forward for a section.  Hence, point must be before the
 section whose type will be found.  Does not move point.  Signal an
 error if the node is not the top node and a section is not found."
-  (let ((case-fold-search t))
-    (save-excursion
-      (cond
-       ((re-search-forward "^@node [ \t]*top[ \t]*\\(,\\|$\\)"
-			   ;; Following search limit by cph but causes a bug
-			   ;;(line-end-position)
-			   nil
-			   t)
-	"top")
-       ((re-search-forward texinfo-section-types-regexp nil t)
-	(buffer-substring-no-properties
-	 (progn (beginning-of-line) ; copy its name
-		(1+ (point)))
-	 (progn (forward-word 1)
-		(point))))
-       (t
-	(error
-	 "texinfo-specific-section-type: Chapter or section not found"))))))
+  (let* ((case-fold-search t)
+	 ;; The Texinfo manual has a second Top node inside @verbatim
+	 ;; near the end, which dupes us into thinking we are at top
+	 ;; level, no matter where we are when invoked.  We don't
+	 ;; really grok @verbatim, so we cheat: only consider us to be
+	 ;; at top level if the position of the Top node we found is
+	 ;; before any other sectioning command.
+	 (top-pos (save-excursion
+		    (re-search-forward "^@node [ \t]*top[ \t]*\\(,\\|$\\)"
+				       ;; Following search limit causes a bug
+				       ;;(line-end-position)
+				       nil
+				       t)))
+	 (sec-pos (save-excursion
+		    (re-search-forward texinfo-section-types-regexp nil t)))
+	 sec-name)
+    (if sec-pos
+	(save-excursion
+	  (goto-char sec-pos)
+	  (setq sec-name (buffer-substring-no-properties
+			  (progn (beginning-of-line) ; copy its name
+				 (1+ (point)))
+			  (progn (forward-word-strictly 1)
+				 (point))))))
+    (cond
+     ((or sec-pos top-pos)
+      (if (and top-pos sec-pos)
+	  (if (< top-pos sec-pos)
+	      "top"
+	    sec-name)
+	(or sec-name "top")))
+     (t
+      (error
+       "texinfo-specific-section-type: Chapter or section not found")))))
 
 (defun texinfo-hierarchic-level ()
   "Return the general hierarchical level of the next node in a texinfo file.
@@ -1358,7 +1375,7 @@ Point must be at beginning of node line.  Does not move point."
   (save-excursion
     (let ((initial (texinfo-copy-next-section-title)))
       ;; This is not clean.  Use `interactive' to read the arg.
-      (forward-word 1)                    ; skip over node command
+      (forward-word-strictly 1)           ; skip over node command
       (skip-chars-forward " \t")          ; and over spaces
       (if (not (looking-at "[^,\t\n ]+")) ; regexp based on what Info looks for
 					  ; alternatively, use "[a-zA-Z]+"
@@ -1391,7 +1408,7 @@ level in the Texinfo file; when looking for the `Previous' pointer,
 the section found will be at the same or higher hierarchical level in
 the Texinfo file; when looking for the `Up' pointer, the section found
 will be at some level higher in the Texinfo file.  The fourth argument
-\(one of 'next, 'previous, or 'up\) specifies whether to find the
+\(one of `next', `previous', or `up') specifies whether to find the
 `Next', `Previous', or `Up' pointer."
   (let ((case-fold-search t))
     (cond ((eq direction 'next)
@@ -1684,7 +1701,7 @@ node names in pre-existing `@node' lines that lack names."
       (if title-p
 	  (progn
 	    (beginning-of-line)
-	    (forward-word 1)
+	    (forward-word-strictly 1)
 	    (skip-chars-forward " \t")
 	    (setq title (buffer-substring
 			 (point)
@@ -1697,7 +1714,7 @@ node names in pre-existing `@node' lines that lack names."
 	       (line-beginning-position -1))
 	   t)
 	  ;;  @node is present, and point at beginning of that line
-	  (forward-word 1)          ; Leave point just after @node.
+	  (forward-word-strictly 1) ; Leave point just after @node.
 	;; Else @node missing; insert one.
 	(beginning-of-line)         ; Beginning of `@section' line.
 	(insert "@node\n")
@@ -1712,7 +1729,7 @@ node names in pre-existing `@node' lines that lack names."
 	    (if (not (looking-at "[^,\t\n ]+"))
 		(progn
 		  (beginning-of-line)
-		  (forward-word 1)
+		  (forward-word-strictly 1)
 		  (insert " " title)
 		  (message "Inserted title %s ... " title)))))
       ;; Go forward beyond current section title.
@@ -1797,7 +1814,7 @@ same place.  If there is no title, returns an empty string."
 	    ;; copy title
 	    (let ((title
 		   (buffer-substring
-		    (progn (forward-word 1)           ; skip over section type
+		    (progn (forward-word-strictly 1)  ; skip over section type
 			   (skip-chars-forward " \t") ; and over spaces
 			   (point))
 		    (progn (end-of-line) (point)))))
@@ -1834,8 +1851,8 @@ chapters."
 
 ;; The menu-list has the form:
 ;;
-;;     \(\(\"node-name1\" . \"title1\"\)
-;;       \(\"node-name2\" . \"title2\"\) ... \)
+;;     ((\"node-name1\" . \"title1\")
+;;      (\"node-name2\" . \"title2\") ... )
 ;;
 ;; However, there does not need to be a title field and this function
 ;; does not fill it; however a comment tells you how to do so.
