@@ -1,7 +1,6 @@
-;;; url-file.el --- File retrieval code
+;;; url-file.el --- File retrieval code  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1996, 1997, 1998, 1999, 2004, 2005, 2006, 2007, 2008,
-;;   2009, 2010, 2011, 2012  Free Software Foundation, Inc.
+;; Copyright (C) 1996-1999, 2004-2021 Free Software Foundation, Inc.
 
 ;; Keywords: comm, data, processes
 
@@ -18,31 +17,31 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
 (require 'mailcap)
 (require 'url-vars)
 (require 'url-parse)
 (require 'url-dired)
+(declare-function mm-disable-multibyte "mm-util" ())
 
 (defconst url-file-default-port 21 "Default FTP port.")
 (defconst url-file-asynchronous-p t "FTP transfers are asynchronous.")
 (defalias 'url-file-expand-file-name 'url-default-expander)
 
-(defun url-file-find-possibly-compressed-file (fname &rest args)
+(defun url-file-find-possibly-compressed-file (fname &rest _)
   "Find the exact file referenced by `fname'.
 This tries the common compression extensions, because things like
 ange-ftp and efs are not quite smart enough to realize when a server
-can do automatic decompression for them, and won't find 'foo' if
-'foo.gz' exists, even though the FTP server would happily serve it up
+can do automatic decompression for them, and won't find `foo' if
+`foo.gz' exists, even though the FTP server would happily serve it up
 to them."
   (let ((scratch nil)
-	(compressed-extensions '("" ".gz" ".z" ".Z" ".bz2"))
+	(compressed-extensions '("" ".gz" ".z" ".Z" ".bz2" ".xz"))
 	(found nil))
     (while (and compressed-extensions (not found))
       (if (file-exists-p (setq scratch (concat fname (pop compressed-extensions))))
@@ -64,14 +63,14 @@ to them."
 						(match-beginning 0))
 				   (system-name)))))))
 
-(defun url-file-asynch-callback (x y name buff func args &optional efs)
+(defun url-file-asynch-callback (_x _y name buff func args &optional efs)
   (if (not (featurep 'ange-ftp))
       ;; EFS passes us an extra argument
       (setq name buff
 	    buff func
 	    func args
 	    args efs))
-  (let ((size (nth 7 (file-attributes name))))
+  (let ((size (file-attribute-size (file-attributes name))))
     (with-current-buffer buff
       (goto-char (point-max))
       (if (/= -1 size)
@@ -90,7 +89,7 @@ to them."
 			    keep-date &optional msg cont nowait))
 
 (defun url-file-build-filename (url)
-  (if (not (vectorp url))
+  (if (not (url-p url))
       (setq url (url-generic-parse-url url)))
   (let* ((user (url-user url))
 	 (pass (url-password url))
@@ -103,21 +102,24 @@ to them."
 		     (format "%s#%d" host port))
 		 host))
 	 (file (url-unhex-string (url-filename url)))
-	 (filename (if (or user (not (url-file-host-is-local-p host)))
-		       (concat "/" (or user "anonymous") "@" site ":" file)
-		     (if (and (memq system-type '(ms-dos windows-nt))
-			      (string-match "^/[a-zA-Z]:/" file))
-			 (substring file 1)
-		       file)))
-	 pos-index)
+	 (filename (cond
+		    ;; ftp: URL.
+		    ((or user (not (url-file-host-is-local-p host)))
+		     (concat "/" (or user "anonymous") "@" site ":" file))
+		    ;; file: URL on Windows.
+		    ((and (string-match "\\`/[a-zA-Z]:/" file)
+			  (memq system-type '(ms-dos windows-nt)))
+		     (substring file 1))
+		    ;; file: URL with a file:/bar:/foo-like spec.
+		    ((string-match "\\`/[^/]+:/" file)
+		     (concat "/:" file))
+		    (t
+		     file))))
 
     (and user pass
 	 (cond
 	  ((featurep 'ange-ftp)
 	   (ange-ftp-set-passwd host user pass))
-	  ((when (featurep 'xemacs)
-             (or (featurep 'efs) (featurep 'efs-auto)
-                 (efs-set-passwd host user pass))))
 	  (t
 	   nil)))
 
@@ -136,17 +138,6 @@ to them."
 	     (not (string-match "/\\'" filename)))
 	(setf (url-filename url) (format "%s/" filename)))
 
-
-    ;; If it is a directory, look for an index file first.
-    (if (and (file-directory-p filename)
-	     url-directory-index-file
-	     (setq pos-index (expand-file-name url-directory-index-file filename))
-	     (file-exists-p pos-index)
-	     (file-readable-p pos-index))
-	(setq filename pos-index))
-
-    ;; Find the (possibly compressed) file
-    (setq filename (url-file-find-possibly-compressed-file filename))
     filename))
 
 ;;;###autoload
@@ -161,24 +152,28 @@ to them."
     (or filename (error "File does not exist: %s" (url-recreate-url url)))
     ;; Need to figure out the content-type from the real extension,
     ;; not the compressed one.
+    ;; FIXME should this regexp not include more extensions; basically
+    ;; everything that url-file-find-possibly-compressed-file does?
     (setq uncompressed-filename (if (string-match "\\.\\(gz\\|Z\\|z\\)$" filename)
 				    (substring filename 0 (match-beginning 0))
 				  filename))
     (setq content-type (mailcap-extension-to-mime
 			(url-file-extension uncompressed-filename))
-	  content-encoding (case (intern (url-file-extension filename))
-			     ((\.z \.gz) "gzip")
-			     (\.Z "compress")
-			     (\.uue "x-uuencoded")
-			     (\.hqx "x-hqx")
-			     (\.bz2 "x-bzip2")
-			     (otherwise nil)))
+	  content-encoding (pcase (url-file-extension filename)
+			     ((or ".z" ".gz") "gzip")
+			     (".Z" "compress")
+			     (".uue" "x-uuencoded")
+			     (".hqx" "x-hqx")
+			     (".bz2" "x-bzip2")
+			     (".xz" "x-xz")
+			     (_ nil)))
 
     (if (file-directory-p filename)
 	;; A directory is done the same whether we are local or remote
 	(url-find-file-dired filename)
       (with-current-buffer
 	  (setq buffer (generate-new-buffer " *url-file*"))
+        (require 'mm-util)
 	(mm-disable-multibyte)
 	(setq url-current-object url)
 	(insert "Content-type: " (or content-type "application/octet-stream") "\n")
@@ -201,19 +196,10 @@ to them."
 	    (if (featurep 'ange-ftp)
 		(ange-ftp-copy-file-internal filename (expand-file-name new) t
 					     nil t
-					     (list 'url-file-asynch-callback
+					     (list #'url-file-asynch-callback
 						   new (current-buffer)
 						   callback cbargs)
-					     t)
-              (when (featurep 'xemacs)
-                (autoload 'efs-copy-file-internal "efs")
-                (efs-copy-file-internal filename (efs-ftp-path filename)
-                                        new (efs-ftp-path new)
-                                        t nil 0
-                                        (list 'url-file-asynch-callback
-                                              new (current-buffer)
-                                              callback cbargs)
-                                        0 nil)))))))
+					     t))))))
     buffer))
 
 (defmacro url-file-create-wrapper (method args)
@@ -234,5 +220,4 @@ to them."
 
 (provide 'url-file)
 
-;; arch-tag: 010e914a-7313-494b-8a8c-6495a862157d
 ;;; url-file.el ends here
