@@ -1,13 +1,14 @@
 /* GNU Emacs routines to deal with case tables.
-   Copyright (C) 1993, 1994, 2001, 2002, 2003, 2004,
-                 2005, 2006, 2007, 2008  Free Software Foundation, Inc.
+   Copyright (C) 1993-1994, 2001-2021 Free Software Foundation, Inc.
+
+Author: Howard Gayle
 
 This file is part of GNU Emacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or (at
+your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,35 +16,26 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.  */
-
-/* Written by Howard Gayle.  */
+along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
+
 #include "lisp.h"
 #include "buffer.h"
-#include "charset.h"
 
-Lisp_Object Qcase_table_p, Qcase_table;
-Lisp_Object Vascii_downcase_table, Vascii_upcase_table;
-Lisp_Object Vascii_canon_table, Vascii_eqv_table;
+Lisp_Object Vascii_downcase_table;
+static Lisp_Object Vascii_upcase_table;
+Lisp_Object Vascii_canon_table;
+static Lisp_Object Vascii_eqv_table;
 
-/* Used as a temporary in DOWNCASE and other macros in lisp.h.  No
-   need to mark it, since it is used only very temporarily.  */
-int case_temp1;
-Lisp_Object case_temp2;
-
-static void set_canon ();
-static void set_identity ();
-static void shuffle ();
+static void set_canon (Lisp_Object case_table, Lisp_Object range, Lisp_Object elt);
+static void set_identity (Lisp_Object table, Lisp_Object c, Lisp_Object elt);
+static void shuffle (Lisp_Object table, Lisp_Object c, Lisp_Object elt);
 
 DEFUN ("case-table-p", Fcase_table_p, Scase_table_p, 1, 1, 0,
        doc: /* Return t if OBJECT is a case table.
 See `set-case-table' for more information on these data structures.  */)
-     (object)
-     Lisp_Object object;
+  (Lisp_Object object)
 {
   Lisp_Object up, canon, eqv;
 
@@ -64,8 +56,7 @@ See `set-case-table' for more information on these data structures.  */)
 }
 
 static Lisp_Object
-check_case_table (obj)
-     Lisp_Object obj;
+check_case_table (Lisp_Object obj)
 {
   CHECK_TYPE (!NILP (Fcase_table_p (obj)), Qcase_table_p, obj);
   return (obj);
@@ -73,20 +64,20 @@ check_case_table (obj)
 
 DEFUN ("current-case-table", Fcurrent_case_table, Scurrent_case_table, 0, 0, 0,
        doc: /* Return the case table of the current buffer.  */)
-     ()
+  (void)
 {
-  return current_buffer->downcase_table;
+  return BVAR (current_buffer, downcase_table);
 }
 
 DEFUN ("standard-case-table", Fstandard_case_table, Sstandard_case_table, 0, 0, 0,
        doc: /* Return the standard case table.
 This is the one used for new buffers.  */)
-     ()
+  (void)
 {
   return Vascii_downcase_table;
 }
 
-static Lisp_Object set_case_table ();
+static Lisp_Object set_case_table (Lisp_Object, bool);
 
 DEFUN ("set-case-table", Fset_case_table, Sset_case_table, 1, 1, 0,
        doc: /* Select a new case table for the current buffer.
@@ -102,31 +93,27 @@ CANONICALIZE maps each character to a canonical equivalent;
  any two characters that are related by case-conversion have the same
  canonical equivalent character; it may be nil, in which case it is
  deduced from DOWNCASE and UPCASE.
-EQUIVALENCES is a map that cyclicly permutes each equivalence class
+EQUIVALENCES is a map that cyclically permutes each equivalence class
  (of characters with the same canonical equivalent); it may be nil,
  in which case it is deduced from CANONICALIZE.  */)
-     (table)
-     Lisp_Object table;
+  (Lisp_Object table)
 {
   return set_case_table (table, 0);
 }
 
-DEFUN ("set-standard-case-table", Fset_standard_case_table, Sset_standard_case_table, 1, 1, 0,
+DEFUN ("set-standard-case-table", Fset_standard_case_table,
+       Sset_standard_case_table, 1, 1, 0,
        doc: /* Select a new standard case table for new buffers.
 See `set-case-table' for more info on case tables.  */)
-     (table)
-     Lisp_Object table;
+  (Lisp_Object table)
 {
   return set_case_table (table, 1);
 }
 
 static Lisp_Object
-set_case_table (table, standard)
-     Lisp_Object table;
-     int standard;
+set_case_table (Lisp_Object table, bool standard)
 {
   Lisp_Object up, canon, eqv;
-  Lisp_Object indices[3];
 
   check_case_table (table);
 
@@ -137,28 +124,29 @@ set_case_table (table, standard)
   if (NILP (up))
     {
       up = Fmake_char_table (Qcase_table, Qnil);
-      map_char_table (set_identity, Qnil, table, table, up, 0, indices);
-      map_char_table (shuffle, Qnil, table, table, up, 0, indices);
-      XCHAR_TABLE (table)->extras[0] = up;
+      map_char_table (set_identity, Qnil, table, up);
+      map_char_table (shuffle, Qnil, table, up);
+      set_char_table_extras (table, 0, up);
     }
 
   if (NILP (canon))
     {
       canon = Fmake_char_table (Qcase_table, Qnil);
-      XCHAR_TABLE (table)->extras[1] = canon;
-      map_char_table (set_canon, Qnil, table, table, table, 0, indices);
+      set_char_table_extras (table, 1, canon);
+      map_char_table (set_canon, Qnil, table, table);
     }
 
   if (NILP (eqv))
     {
       eqv = Fmake_char_table (Qcase_table, Qnil);
-      map_char_table (set_identity, Qnil, canon, canon, eqv, 0, indices);
-      map_char_table (shuffle, Qnil, canon, canon, eqv, 0, indices);
-      XCHAR_TABLE (table)->extras[2] = eqv;
+      map_char_table (set_identity, Qnil, canon, eqv);
+      map_char_table (shuffle, Qnil, canon, eqv);
+      set_char_table_extras (table, 2, eqv);
     }
 
-  /* This is so set_image_of_range_1 in regex.c can find the EQV table.  */
-  XCHAR_TABLE (canon)->extras[2] = eqv;
+  /* This is so set_image_of_range_1 in regex-emacs.c can find the EQV
+     table.  */
+  set_char_table_extras (canon, 2, eqv);
 
   if (standard)
     {
@@ -169,10 +157,10 @@ set_case_table (table, standard)
     }
   else
     {
-      current_buffer->downcase_table = table;
-      current_buffer->upcase_table = up;
-      current_buffer->case_canon_table = canon;
-      current_buffer->case_eqv_table = eqv;
+      bset_downcase_table (current_buffer, table);
+      bset_upcase_table (current_buffer, up);
+      bset_case_canon_table (current_buffer, canon);
+      bset_case_eqv_table (current_buffer, eqv);
     }
 
   return table;
@@ -180,30 +168,45 @@ set_case_table (table, standard)
 
 /* The following functions are called in map_char_table.  */
 
-/*  Set CANON char-table element for C to a translated ELT by UP and
-   DOWN char-tables.  This is done only when ELT is a character.  The
-   char-tables CANON, UP, and DOWN are in CASE_TABLE.  */
+/* Set CANON char-table element for characters in RANGE to a
+   translated ELT by UP and DOWN char-tables.  This is done only when
+   ELT is a character.  The char-tables CANON, UP, and DOWN are in
+   CASE_TABLE.  */
 
 static void
-set_canon (case_table, c, elt)
-     Lisp_Object case_table, c, elt;
+set_canon (Lisp_Object case_table, Lisp_Object range, Lisp_Object elt)
 {
   Lisp_Object up = XCHAR_TABLE (case_table)->extras[0];
   Lisp_Object canon = XCHAR_TABLE (case_table)->extras[1];
 
-  if (NATNUMP (elt))
-    Faset (canon, c, Faref (case_table, Faref (up, elt)));
+  if (FIXNATP (elt))
+    Fset_char_table_range (canon, range, Faref (case_table, Faref (up, elt)));
 }
 
-/* Set elements of char-table TABLE for C to C itself.  This is done
-   only when ELT is a character.  This is called in map_char_table.  */
+/* Set elements of char-table TABLE for C to C itself.  C may be a
+   cons specifying a character range.  In that case, set characters in
+   that range to themselves.  This is done only when ELT is a
+   character.  This is called in map_char_table.  */
 
 static void
-set_identity (table, c, elt)
-     Lisp_Object table, c, elt;
+set_identity (Lisp_Object table, Lisp_Object c, Lisp_Object elt)
 {
-  if (NATNUMP (elt))
-    Faset (table, c, c);
+  if (FIXNATP (elt))
+    {
+      int from, to;
+
+      if (CONSP (c))
+	{
+	  from = XFIXNUM (XCAR (c));
+	  to = XFIXNUM (XCDR (c));
+	}
+      else
+	from = to = XFIXNUM (c);
+
+      to++;
+      for (; from < to; from++)
+	CHAR_TABLE_SET (table, from, make_fixnum (from));
+    }
 }
 
 /* Permute the elements of TABLE (which is initially an identity
@@ -212,66 +215,80 @@ set_identity (table, c, elt)
    operated.  */
 
 static void
-shuffle (table, c, elt)
-     Lisp_Object table, c, elt;
+shuffle (Lisp_Object table, Lisp_Object c, Lisp_Object elt)
 {
-  if (NATNUMP (elt) && !EQ (c, elt))
+  if (FIXNATP (elt))
     {
-      Lisp_Object tem = Faref (table, elt);
-      Faset (table, elt, c);
-      Faset (table, c, tem);
+      int from, to;
+
+      if (CONSP (c))
+	{
+	  from = XFIXNUM (XCAR (c));
+	  to = XFIXNUM (XCDR (c));
+	}
+      else
+	from = to = XFIXNUM (c);
+
+      to++;
+      for (; from < to; from++)
+	{
+	  Lisp_Object tem = Faref (table, elt);
+	  Faset (table, elt, make_fixnum (from));
+	  Faset (table, make_fixnum (from), tem);
+	}
     }
 }
 
 void
-init_casetab_once ()
+init_casetab_once (void)
 {
   register int i;
-  Lisp_Object down, up;
-  Qcase_table = intern ("case-table");
-  staticpro (&Qcase_table);
+  Lisp_Object down, up, eqv;
 
-  /* Intern this now in case it isn't already done.
-     Setting this variable twice is harmless.
-     But don't staticpro it here--that is done in alloc.c.  */
-  Qchar_table_extra_slots = intern ("char-table-extra-slots");
-
-  /* Now we are ready to set up this property, so we can
-     create char tables.  */
-  Fput (Qcase_table, Qchar_table_extra_slots, make_number (3));
+  DEFSYM (Qcase_table, "case-table");
+  Fput (Qcase_table, Qchar_table_extra_slots, make_fixnum (3));
 
   down = Fmake_char_table (Qcase_table, Qnil);
   Vascii_downcase_table = down;
-  XCHAR_TABLE (down)->purpose = Qcase_table;
+  set_char_table_purpose (down, Qcase_table);
 
-  for (i = 0; i < CHAR_TABLE_SINGLE_BYTE_SLOTS; i++)
-    XSETFASTINT (XCHAR_TABLE (down)->contents[i],
-		 (i >= 'A' && i <= 'Z') ? i + ('a' - 'A') : i);
+  for (i = 0; i < 128; i++)
+    {
+      int c = (i >= 'A' && i <= 'Z') ? i + ('a' - 'A') : i;
+      CHAR_TABLE_SET (down, i, make_fixnum (c));
+    }
 
-  XCHAR_TABLE (down)->extras[1] = Fcopy_sequence (down);
+  set_char_table_extras (down, 1, Fcopy_sequence (down));
 
   up = Fmake_char_table (Qcase_table, Qnil);
-  XCHAR_TABLE (down)->extras[0] = up;
+  set_char_table_extras (down, 0, up);
 
-  for (i = 0; i < CHAR_TABLE_SINGLE_BYTE_SLOTS; i++)
-    XSETFASTINT (XCHAR_TABLE (up)->contents[i],
-		 ((i >= 'A' && i <= 'Z')
-		  ? i + ('a' - 'A')
-		  : ((i >= 'a' && i <= 'z')
-		     ? i + ('A' - 'a')
-		     : i)));
+  for (i = 0; i < 128; i++)
+    {
+      int c = (i >= 'a' && i <= 'z') ? i + ('A' - 'a') : i;
+      CHAR_TABLE_SET (up, i, make_fixnum (c));
+    }
 
-  XCHAR_TABLE (down)->extras[2] = Fcopy_sequence (up);
+  eqv = Fmake_char_table (Qcase_table, Qnil);
+
+   for (i = 0; i < 128; i++)
+     {
+      int c = ((i >= 'A' && i <= 'Z') ? i + ('a' - 'A')
+	       : ((i >= 'a' && i <= 'z') ? i + ('A' - 'a')
+		  : i));
+      CHAR_TABLE_SET (eqv, i, make_fixnum (c));
+    }
+
+  set_char_table_extras (down, 2, eqv);
 
   /* Fill in what isn't filled in.  */
   set_case_table (down, 1);
 }
 
 void
-syms_of_casetab ()
+syms_of_casetab (void)
 {
-  Qcase_table_p = intern ("case-table-p");
-  staticpro (&Qcase_table_p);
+  DEFSYM (Qcase_table_p, "case-table-p");
 
   staticpro (&Vascii_canon_table);
   staticpro (&Vascii_downcase_table);
@@ -284,6 +301,3 @@ syms_of_casetab ()
   defsubr (&Sset_case_table);
   defsubr (&Sset_standard_case_table);
 }
-
-/* arch-tag: e06388ad-99fe-40ec-ba67-9d010fcc4916
-   (do not change this comment) */
